from zope import component
from hurry.resource.interfaces import ICurrentNeededInclusions
from hurry.resource import Library, ResourceInclusion

foo = Library("foo")

a = ResourceInclusion(foo, "a.js")

b = ResourceInclusion(foo, "b.js", depends=[a])

class TestSingle(object):
    def widget(self):
        a.need()
        return "the widget HTML itself"

class TestMultiple(object):
    def widget(self):
        b.need()
        return "the widget HTML itself"



# zope integration for hurry.resource
from grokcore import component as grok

from zope import interface
from zope import component
import zope.security.management
from zope.publisher.interfaces import IRequest
from zope.app.component.hooks import getSite
from zope.traversing.browser.interfaces import IAbsoluteURL
from zope.publisher.browser import BrowserRequest, BrowserResponse, isHTML
from zope.app.publication.interfaces import IBrowserRequestFactory

from hurry.resource import NeededInclusions, Library
from hurry.resource.interfaces import ICurrentNeededInclusions, ILibraryUrl

class CurrentNeededInclusions(grok.GlobalUtility):
    grok.implements(ICurrentNeededInclusions)
    grok.provides(ICurrentNeededInclusions)
    
    def __call__(self):
        request = getRequest()
        if not hasattr(request, 'hurry_resource_needed'):
            request.hurry_resource_needed = NeededInclusions()
        return request.hurry_resource_needed

@grok.adapter(Library)
@grok.implementer(ILibraryUrl)
def library_url(library):
    request = getRequest()
    return str(component.getMultiAdapter((getSite(), request),
                                         IAbsoluteURL)) + '/@@/' + library.name

class Request(BrowserRequest):
    interface.classProvides(IBrowserRequestFactory)

    def _createResponse(self):
        return Response()

class Response(BrowserResponse):
    def _implicitResult(self, body):
        content_type = self.getHeader('content-type')
        if content_type is None:
            if isHTML(body):
                content_type = 'text/html'
            else:
                content_type = 'text/plain'
            self.setHeader('x-content-type-warning', 'guessed from content')
            self.setHeader('content-type', content_type)
        
        # check the content type disregarding parameters and case
        if content_type and content_type.split(';', 1)[0].lower() in (
            'text/html', 'text/xml'):
            # act on HTML and XML content only!

            needed = component.getUtility(ICurrentNeededInclusions)()

            html = needed.render()
            
            if html:
                body = body.replace('<head>', '<head>\n    %s\n' %
                                    html, 1)
        return super(Response, self)._implicitResult(body)

class NoRequestError(Exception):
    pass

def getRequest():
    try:
        i = zope.security.management.getInteraction() # raises NoInteraction
    except zope.security.interfaces.NoInteraction:
        raise NoRequestError()

    for p in i.participations:
        if IRequest.providedBy(p):
            return p

    raise NoRequestError()



import os
import unittest
import doctest

from zope.app.testing.functional import FunctionalDocFileSuite
from zope.app.testing import functional

FunctionalLayer = functional.ZCMLLayer(
    os.path.join(os.path.split(__file__)[0], 'ftesting.zcml'),
    __name__, 'FunctionalLayer')

def test_suite():
    optionflags = doctest.NORMALIZE_WHITESPACE | doctest.ELLIPSIS
    globs = {}
    
    suite = unittest.TestSuite()
    
    readme = FunctionalDocFileSuite(
        '../README.txt',
        optionflags=optionflags,
        globs=globs)

    readme.layer = FunctionalLayer
    
    suite.addTest(readme)

    return suite
